using System;

namespace RFIDForm
{	

	//^Õf[^p\
	public struct RFIDDataRx
	{
		public long id;
		public int type;
	}


	public enum IdTx{Read, ReadLine, Program};

	//RFID[pNX
	public class RFIDComm
	{
		//RFID[_[ɑM\
		private struct RFIDCommandTx	//Max Length = 41
		{
			public byte start;	
			public byte length; 
			public byte command1;
			public byte command2;
			public byte [] data; 
			public byte bcc;
		}
		//RFID[_[M\
		private struct RFIDCommandRx
		{
			public byte start;
			public byte length;
			public byte status;
			public byte [] data;
			public byte bcc;

		}

		//RFID[_[Ƃ̒ʐMʂ萔
		public const int RX_ID_ERROR_READ = -2;
		public const int RX_ID_OTHER_READ = -1;
		public const int RX_ID_NO_READ = 0;
		//^O̎ނ萔
		public const int RX_TYPE_RO	= 0x00;
		public const int RX_TYPE_RW	= 0x01;
		public const int RX_TYPE_MPT	= 0x02;
		public const int RX_TYPE_OTHER = 0x03;

		//ʐMp̃X^[goCg
		const byte SOH = 0x01;

		//RFIDւ̖߂\萔
		const byte READ_SINGLE = 0x08;
		const byte READ_NORMAL = 0x09;
		const byte READ_LINE = 0x0A;	
		const byte PROGRAM_SINGLE = 0xE8;	//1110 1000[Power Burst1&2 with Data, Single, No FBCC, CommandField2]
		const byte PROGRAM_SINGLE2 = 0x06;	// 0000 0110Wireless Synchronization, Calcurate DBCC
	
		//VA|[gʐMpNX̐錾
		Rs232c comm;

		//RXgN^
		public RFIDComm(string port, int baudRate)
		{		
			comm = new Rs232c(port, baudRate);
		}

		//VAʐMp|[gԍݒC擾
		public string Port
		{
			set{comm.Port = value;}
			get{return comm.Port;}
		}

		//VAʐM|[gJ
		public bool Open()
		{
			bool flag = comm.Open();
			return flag;
		}

		//VAʐM|[g
		public void Close()
		{
			comm.Close();
			return;
		}

		//VAʐM|[gJĂtrueCĂfalse
		public bool IsOpen
		{
			get{return this.comm.IsOpen;}
		}


		//RFID[_ɃbZ[W𑗐M
		public bool SendMessage(IdTx id, ref byte [] txBytes)
		{
			byte []data = new byte[1];
			return SendMessage(id, data,ref txBytes);
		}

		//VA|[gʂāCRFID[_ɃbZ[W𑗐M
		public bool SendMessage(IdTx id, byte [] data, ref byte [] txBytes)
		{

			RFIDCommandTx commandTx;

			commandTx = new RFIDCommandTx();
			commandTx.start = SOH;

			switch(id)
			{
				case IdTx.Read:
					commandTx.length  = 2;
					commandTx.command1 = READ_SINGLE;
					commandTx.data = new byte[1];
					commandTx.data[0] = 50;
					commandTx.bcc = (byte)(commandTx.length ^ commandTx.command1 ^ commandTx.data[0]);
					
					break;
				case IdTx.ReadLine:
					commandTx.length  = 2;
					commandTx.command1 = READ_LINE;
					commandTx.data = new byte[1];
					commandTx.data[0] = 50;
					commandTx.bcc = (byte)(commandTx.length ^ commandTx.command1 ^ commandTx.data[0]);

					break;
				case IdTx.Program:
					commandTx.length = 17;
					commandTx.command1 = PROGRAM_SINGLE;
					commandTx.command2 = PROGRAM_SINGLE2;
					commandTx.data = new byte[15];
					commandTx.data[0] = 50;		//Power Burst1
					commandTx.data[1] = 15;		//Power Burst2
					commandTx.data[2] = 12;		//Data Fields Follow
					commandTx.data[3] = 0xBB;	//Write Keyword
					commandTx.data[4] = 0xEB;	//Write Password
					for(int i = 0;i< 8;i++)		//Write Data(LSByte First)
					{
						commandTx.data[i+5] = data[i];
					}

					commandTx.data[13] = 0x00;	//Write Frame
					commandTx.data[14] = 0x03;  //Write Frame

					commandTx.bcc = (byte)(commandTx.length ^ commandTx.command1 ^ commandTx.command2);
					for(int i=0;i<15;i++){
						commandTx.bcc = (byte)(commandTx.bcc ^ commandTx.data[i]);
					}
					break;
				default:
					return false;
			}

			//\̂oCgɕϊ
			txBytes = ConvertToBytes(commandTx);
			//VA|[gɃoCgo
			comm.WriteBytes(txBytes);

			return true;
		}

		//VA|[gʂāCRFID[_烁bZ[WMCID擾
		public bool ReceiveMessage(ref byte [] rxBytes, ref RFIDDataRx dataRx)
		{
			if(!comm.IsOpen)
				return false;		
	
			bool flag = comm.ReadBytes(ref rxBytes);
			
			if(!flag)
				return false;

			//̓f[^̔
			RFIDCommandRx commandRx;

			if(rxBytes[0] != RFIDComm.SOH)
			{
				dataRx.id = RFIDComm.RX_ID_ERROR_READ;
				return false;
			}

			if(rxBytes[1] == 1)//No Read
			{
				if(rxBytes.Length < 4)
				{
					dataRx.id = RFIDComm.RX_ID_ERROR_READ;
					return false;
				}

				commandRx.start = rxBytes[0];
				commandRx.length = rxBytes[1];
				commandRx.status = rxBytes[2];
				commandRx.bcc = rxBytes[3];
				dataRx.id = RFIDComm.RX_ID_NO_READ;
				return true;
			}
			else if(rxBytes[1] == 9)
			{
				if(rxBytes.Length < 11)
				{
					dataRx.id = RFIDComm.RX_ID_ERROR_READ;
					return false;
				}
				commandRx.start = rxBytes[0];
				commandRx.length = rxBytes[1];
				commandRx.status = rxBytes[2];
				commandRx.data = new byte[8];
				for(int i=0;i<8;i++)
				{
					commandRx.data[i] = rxBytes[i+3];
				}
				commandRx.bcc = rxBytes[11];
				dataRx.type = commandRx.status & 0x03;				//rbgi[
				dataRx.id = BitConverter.ToInt64(commandRx.data,0);
				return true;
			}
			else
			{
				if(rxBytes.Length < 3)
				{
					dataRx.id = RFIDComm.RX_ID_ERROR_READ;
					return false;
				}

				dataRx.type = rxBytes[2] & 0x03;
				dataRx.id = RFIDComm.RX_ID_OTHER_READ;
				return true;
			}
		}

		
		//RFIDCommandTx^\̂oCgɕϊ
		private byte [] ConvertToBytes(RFIDCommandTx tx)
		{
			byte [] txBytes = new byte[3 + tx.length];
			txBytes[0] = tx.start;
			txBytes[1] = tx.length;
			txBytes[2] = tx.command1;
			
			if(tx.command2 != 0)
			{
				txBytes[3] = tx.command2;
				for(int i=0;i<tx.length-2;i++)
				{
					txBytes[i+4] = tx.data[i];
				}
			}

			else
			{
				for(int i=0;i<tx.length-1;i++)
				{
					txBytes[i+3] = tx.data[i];
				}

			}
			txBytes[2 + tx.length] = tx.bcc;
			return txBytes;
		}
		
	}
}
